%   This is really a multiple binomial logit model
%   The model is
%       z(:,k) = x * beta(k) + u(:,k),    for n = 1,...,N ; k = 1,...,K
%           for identification we set beta(K+1) = 0
%       u(n,k) ~ N(0,lam(n,k)) and independent across n
%   In this model we allow the possibility that state n belongs to more than one classification
%       Thus, the probability that state n belongs to classification j is
%       Pr(Y(n) = j | beta(j)) = exp(x(n) * beta(j) / 1 + exp(x(n) * beta(j))
%   This quanity is independent of whether state n belongs to any other classification
%       Here, beta(i) is a P x 1 vector of coefficients
%       lam(i) is a K x 1 vector of variances

%   This program estimates the model without covariate uncertainty
%       and with a fixed number of clusters


function [bta,lam,z] = HO_MNL_CV(priors,draw)

bta = zeros(priors.pp,priors.no_clust);
z = zeros(priors.N,priors.no_clust);
lam = zeros(priors.N,priors.no_clust);
OK_OK = 0;
while OK_OK == 0;
    for k = 1:priors.no_clust
        %*******************************************
        %   BETA Block
        %*******************************************
        %   Compute the hyperparameters for the normal distribution based on the prior N(b,B)
        %   For this part, we're drawing each beta(k) individually
        %   The posterior for beta(k) is conditionally normal with hyperparameters
        %       B = 1 / (inv(priors.B) + x' * W * x)
        %       b = B * (priors.B * priors.b + x' * W * draw.z(k,:))
        W = inv(diag(draw.lam(:,k)));
        B = inv(inv(priors.B) + priors.x' * W * priors.x);
        b = B * (inv(priors.B) * priors.b(:,k) + priors.x' * W * draw.z(:,k));
        bta(:,k) = b + chol(B,'lower') * normrnd(0,1,priors.pp,1);

        %*******************************************
        %   Z, LAM Block
        %*******************************************
        %   z is defined such that
        %       if y(n,k) = 1, z(n,k) > 0
        %       if y(n,k) = 0, z(n,k) <= 0
        %   Then, we can draw z(n,k) from a truncated logistic(x * beta)

        for n = 1:priors.N;
            %*******************************************
            %   Drawing Z from a logistic distribution
            %*******************************************
            A = priors.x(n,:) * bta(:,k);
            acc = 0;count = 0;
            while acc == 0
                u_star = rand;
                if draw.KAY(n,k) == 0
                    u = u_star / (1 + exp(A));
                else
                    u = (1 / (1 + exp(A))) + (exp(A) * u_star) / (1 + exp(A));
                end,
                zz = A - log( inv(u) - 1 );
                if isinf(zz) == 1
                    if count > 1000
                        if draw.KAY(n,k) == 0
                            zz = -0.000001;
                        else
                            zz = 0.000001;
                        end
                        acc = 1;
                    end
                else
                    acc = 1;
                end,
            end,
            z(n,k) = zz;
            %*******************************************
            %   rejection sampler for lambda
            %*******************************************
            OK = 0;
            count = 0;
            while OK == 0
                %   draw a candidate from the GIG(0.5,1,r^2)  -- check to see if this is r or r_sq)
                r_sq = (draw.z(n,k) - priors.x(n,:) * bta(:,k)) ^ 2;
                r = sqrt(r_sq);
                %v = chi2rnd(1,1);
                VV = normrnd(0,1);
                v = VV ^ 2;
                g = 1 + ((v - sqrt(4 * v * r + v ^ 2)) / (2 * r));
                u = rand;
                if u <= inv(1 + g)
                    lam_hat = r / g;
                else
                    lam_hat = r * g;
                end
                U = rand;
                %   can't compute the acceptance probability directly so split the function in two
                if lam_hat > 4/3
                    [OK] = right_interval(U,lam_hat);
                    if OK == 1
                        OK_OK = 1;
                    end,
                else
                    [OK] = left_interval(U,lam_hat);
                    if OK == 1
                        OK_OK = 1;
                    end,
                end,
                count = count + 1;
                if count > 1000
                    OK = 1;
                end,
            end,
            lam(n,k) = lam_hat;
        end,
    end,
end,